NinjaPoints = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0")

local self = NinjaPoints

local LSM = AceLibrary("LibSharedMedia-3.0")
local dewdrop = AceLibrary("Dewdrop-2.0")

NinjaPoints.string = UIParent:CreateFontString("NinjaPointsString", "OVERLAY")

local function CreateTable(index)
	local newTable = {
		type = 'group',
		name = tostring(index),
		desc = 'Configure options for combo point '..string.upper(tostring(index)),
		order = tonumber(index),
		args = {
			text = {
				type = 'text',
				name = 'Text',
				desc = 'What text you want combo point '..string.upper(tostring(index))..' to show',
				get = function() return self.db.profile.texts[index] end,
				set = function(v) self.db.profile.texts[index] = v end,
				usage = "<string>",
			},
			style = {
				type = 'text',
				name = 'Style',
				desc = 'Style to show combo points in',
				get = function() return self.db.profile.style[index] end,
				set = function(v) self.db.profile.style[index] = v end,
				validate = { "number", "text", "random" },
			}, 
			randomColor = {
				type = 'toggle',
				name = 'randomColor',
				desc = 'Chooses a random color to show for combo ponit '..string.upper(tostring(index)),
				get = function() return self.db.profile.randomColor[index] end,
				set = function(v) self.db.profile.randomColor[index] = v end,
			},
			color = {
				type = 'color',
				name = 'Color',
				desc = 'Change the color of combo point '..string.upper(tostring(index))..' text',
				get = function() return self.db.profile.colors[index].red, self.db.profile.colors[index].green, self.db.profile.colors[index].blue end,
				set = function(r, g, b)
					self.db.profile.colors[index].red = r
					self.db.profile.colors[index].green = g
					self.db.profile.colors[index].blue = b
				end,
			},
			sound = {
				type = 'text',
				name = 'Sound',
				desc = 'Play a sound file when you obtain combo point '..string.upper(tostring(index))..' (Default: None)',
				get = function() return self.db.profile.sounds[index] end,
				set = function(v)
					self.db.profile.sounds[index] = v
					PlaySoundFile(LSM:Fetch('sound', v))
				end,
				validate = LSM:List('sound'),
			},
		},
	}
	return newTable
end

local options = {
	type = 'group',
	name = 'NinjaPoints',
	desc = 'Displays combo points in large, annoying numbers or words',
	args = {
		font = {
			type = 'group',
			name = 'Font',
			desc = 'Change font options',
			order = 1,
			args = {
				face = {
					type = 'text',
					name = 'Face',
					desc = 'Set combo point text font face',
					get = function() return self.db.profile.fontFace end,
					set = function(v) self.db.profile.fontFace = v end,
					validate = LSM:List('font'),
					order = 1,
				},
				size = {
					type = 'range',
					name = 'Size',
					desc = 'Set combo point font size',
					get = function() return self.db.profile.fontSize end,
					set = function(v) self.db.profile.fontSize = v end,
					min = 18,
					max = 72,
					step = 1,
					order = 2,
				},
				outline = {
					type = 'text',
					name = 'Outline',
					desc = 'Set font outline',
					get = function() return self.db.profile.fontOutline end,
					set = function(v) self.db.profile.fontOutline = v end,
					order = 3,
					validate = { "NONE", "MONOCHROME", "OUTLINE", "THICKOUTLINE" },
				},
			},
		},
		test = {
			type = 'execute',
			name = 'Test',
			desc = 'Test combo point text frame',
			func = "ToggleTestFrame",
			order = 7,
		},
		points = {
			type = 'group',
			name = 'Points',
			desc = 'Configure what to show for combo points',
			args = {
				one = CreateTable(1),
				two = CreateTable(2),
				three = CreateTable(3),
				four = CreateTable(4),
				five = CreateTable(5),
			},
			order = 3,
		},
		xOff = {
			type = 'range',
			name = 'xOff',
			desc = 'Set X offset for combo point text',
			get = function()
				local _, _, _, x, y = self.string:GetPoint()
				return x
			end,
			set = function(v)
				local a, b, c, x, y = self.string:GetPoint()
				self.db.profile.xOff = v
				self.string:SetPoint(a, b, c, self.db.profile.xOff, y)
				self.string:Hide()
				self:ToggleTestFrame()
			end,
			min = -UIParent:GetWidth(),
			max = UIParent:GetWidth(),
			order = 4,
		},
		yOff = {
			type = 'range',
			name = 'yOff',
			desc = 'Set Y offset for combo point text',
			order = 5,
			get = function()
				local _, _, _, x, y = self.string:GetPoint()
				return y
			end,
			set = function(v)
				local a, b, c, x, y = self.string:GetPoint()
				self.db.profile.yOff = v
				self.string:SetPoint(a, b, c, x, self.db.profile.yOff)
				self.string:Hide()
				self:ToggleTestFrame()
			end,
			min = -UIParent:GetHeight(),
			max = UIParent:GetHeight(),
		},
		config = {
			type = 'execute',
			name = 'Config',
			desc = 'Open Dewdrop config menu',
			func = "OpenConfig",
			hidden = function() return dewdrop:IsOpen(self.string) end,
		},
		reset = {
			type = 'execute',
			name = 'Reset',
			desc = 'Reset options back to their defaults',
			order = 8,
			func = "ResetDB",
			confirm = true,
		},
		spacer = {
			type = 'header',
			order = 6,
			hidden = function() return not dewdrop:IsOpen(self.string) end,
		},
	},
}

local defaultDB = {
	fontFace = "Friz Quadrata TT",
	fontSize = "36",
	fontOutline = "NORMAL",
	xOff = 0,
	yOff = 400,
	style = {
		[1] = "number",
		[2] = "number",
		[3] = "number",
		[4] = "number",
		[5] = "number",
	},
	randomColor = {
		[1] = false,
		[2] = false,
		[3] = false,
		[4] = false,
		[5] = false,
	},
	sounds = {
		[1] = "None",
		[2] = "None",
		[3] = "None",
		[4] = "None",
		[5] = "None",
	},
	texts = {
		[1] = "ONE",
		[2] = "TWO",
		[3] = "THREE",
		[4] = "FOUR",
		[5] = "FIVE",
	},
	colors = {
		[1] = {
			red = ".17",
			green = ".90",
			blue = ".11",
		},
		[2] = {
			red = ".18",
			green = ".93",
			blue = ".93",
		},
		[3] = {
			red = ".89",
			green = ".93",
			blue = ".18",
		},
		[4] = {
			red = ".22",
			green = ".18",
			blue = ".80",
		},
		[5] = {
			red = "1.00",
			green = ".0",
			blue = ".0",
		},
	}
}

NinjaPoints.randomMessages = {
	"Only "..GetComboPoints().." points? Quit slacking!",
	"Bust out that ninjitsu "..UnitName("player").."-san",
	"Make me proud "..UnitName("player"),
	"Go go 800 eviscerate crit!",
	"/\\/\\( \',,\' )/\\/\\",
	"ENVENOM!!1",
}

function NinjaPoints:OnInitialize()
	self:RegisterDB("NinjaPointsDB")
	self:RegisterDefaults("profile", defaultDB)
	
	self:RegisterChatCommand("/npoints", options)
	self:RegisterChatCommand("/np", options)

	if LSM then
		LSM:Register("sound", "Finish Him!", "Interface\\AddOns\\NinjaPoints\\finishim.wav")
	end
	
	self.string:SetFont(LSM:Fetch('font', self.db.profile.fontFace), self.db.profile.fontSize, self.db.profile.fontOutline)
	self.string:SetPoint("CENTER", UIParent, self.db.profile.xOff, self.db.profile.yOff)
	self.string:Hide()
end

function NinjaPoints:OnEnable()
	self:RegisterEvent("PLAYER_COMBO_POINTS")
end

function NinjaPoints:PLAYER_COMBO_POINTS()
	local points = GetComboPoints()
	if ( points == 0 ) then self.string:Hide(); return end
	local r, g, b = self:GetTextColors(points)

	self.string:SetFont(LSM:Fetch('font', self.db.profile.fontFace), self.db.profile.fontSize, self.db.profile.fontOutline)
	if self.db.profile.style[points] == "number" then
		self.string:SetText(points)
	elseif self.db.profile.style[points] == "random" then
		self.string:SetText(self.randomMessages[math.random(#self.randomMessages)])
	elseif self.db.profile.style[points] == "text" then
		self.string:SetText(self.db.profile.texts[points])
	end

	self.string:SetTextColor(r, g, b, 1.0)
	self.string:Show()

	PlaySoundFile(LSM:Fetch('sound', self.db.profile.sounds[points]))
end

function NinjaPoints:ResetDB()
	self.db.profile = defaultDB
	self.string:SetPoint("CENTER", UIParent, self.db.profile.xOff, self.db.profile.yOff)
	self:Print("NinjaPointsDB reset to original values")
end

function NinjaPoints:ToggleTestFrame()
	if self.string:IsVisible() and GetComboPoints() == 0 then
		self.string:Hide()
	else
		local point = math.floor(math.random(1, 5))
		local r, g, b = self:GetTextColors(point)
		self.string:SetFont(LSM:Fetch('font', self.db.profile.fontFace), self.db.profile.fontSize, self.db.profile.fontOutline)	
		if self.db.profile.style[point] == "number" then
			self.string:SetText(point)
		elseif self.db.profile.style[point] == "random" then
			self.string:SetText(self.randomMessages[math.random(#self.randomMessages)])
		elseif self.db.profile.style[point] == "text" then
			self.string:SetText(self.db.profile.texts[point])
		end

		self.string:SetTextColor(r, g, b, 1.0)
		self.string:Show()

		PlaySoundFile(LSM:Fetch('sound', self.db.profile.sounds[point]))
	end
end

function NinjaPoints:GetTextColors(index)
	local r, g, b
	if self.db.profile.randomColor[index] then
		r = math.random()
		g = math.random()
		b = math.random()
	else
		r = self.db.profile.colors[index].red
		g = self.db.profile.colors[index].green
		b = self.db.profile.colors[index].blue
	end
	return r, g, b
end

function NinjaPoints:OpenConfig()
	dewdrop:Open(self.string,
		'children', function()
			dewdrop:FeedAceOptionsTable(options)
		end
	)
end